import csv
import numpy as np
import pandas as pd
import math
from collections import deque
from itertools import chain

#TODO: this has been set to a dummy week value. we need a better way to do this than globals
BAR_WIDTH = (60*24*7) #in minutes
TIME_BACK = 1 #in weeks
MARKET_HOURS = 5.5 #number of trading hours in a day
MARKET_DAYS = 5#number of trading days in a week

class CSVWriter():
    def __init__(self, dir = '../data/backtest.csv'):
        self.dir = dir

    def initCSV(self, strategies, indicators, price_days, type, categories, pooling=False):

        if pooling:
            price_days = price_days/4

        header = ['TYPE', 'CATEGORIES', 'INPUT_VECTOR_COL', 'OUTPUT_VECTOR_COL', "DATA_LENGTH"]

        _type = type
        _categories = 0
        if type != 'regression':
            _type = 'classification'
            _categories = categories

        header_data = [_type, _categories, 2]
        s = []

        s += ['Number']
        s += ['Stock']
        s += ['Date']

        for i in range(0,price_days):
            s += ['Price {}'.format(i)]
        for indicator in indicators:
            s += [indicator]
        for strategy in strategies:
            s += [strategy.name + 'Performance']
            s += [strategy.name + 'WinRatio']
            s += [strategy.name + 'AvgWinPct']
            s += [strategy.name + 'AvgLossPct']
            s += [strategy.name + 'AvgDrawdown']
            s += [strategy.name + 'KRatio']
        header_data += [len(s)]
        for strategy in strategies:
            s += [strategy.name + 'Weight']
        header_data += [len(s)]

        with open(self.dir, 'a') as f:
            writer = csv.writer(f, delimiter = ',')
            writer.writerow(header)
            writer.writerow(header_data)
            writer.writerow(s)

    def appendToCSV(self, data, stock):
        fields = data.tolist()

        with open(self.dir, 'r') as f:
            reader = csv.reader(f, delimiter = ',')
            try:
                lastrow = deque(reader, 1)[0]
            except IndexError:  # empty file
                lastrow = None

        if lastrow[0] == 'Number':
            index = 0
        else:
            index = int(lastrow[0]) + 1

        fields = [index, stock] + fields

        with open(self.dir, 'a') as f:
            writer = csv.writer(f, delimiter = ',')
            writer.writerow(fields)

    #TODO: this needs to reference the header
    def fetchExample(self, index):

        ex_array = self.fetchLineFromCSV(index)

        with open(self.dir, 'ab') as f:
            reader = csv.reader(f, delimiter=',')
            out_index = (reader[1])[3]
        return ex_array[0:out_index-1], ex_array[out_index:]

    def fetchLineFromCSV(self, index):

        with open(self.dir, 'ab') as f:
            reader = csv.reader(f, delimiter=',')
            data = (reader[index])[2:]

        return np.asarray(data)


