from zipline.pipeline.factors import ExponentialWeightedMovingAverage, RSI

from bt.pipeline_factors.custom_factors import *
from bt.strategies.abs_strategy import Strategy
from zipline.api import attach_pipeline, pipeline_output, record, symbol
from zipline.pipeline import CustomFactor, Pipeline
from zipline.pipeline.data import USEquityPricing
from bt.backtest.lstm.data_retrieve import CSVWriter
from bt.backtest.lstm.normalizer import normalize_file
from zipline.utils.calendars import get_calendar
import skimage.measure

import numpy as np
import pandas as pd
import copy as cp
import math


def generate(assets, strategy, indicators, num_intervals, type, dir, input_td = 40, output_td = 40, categories=None, pooling=False, append=False):
    # currently expects strategies to have the correct starting and ending dates for the first interval

    start_params = cp.deepcopy(strategy.params)

    generator_params = cp.deepcopy(start_params)
    generator_params.update(td=1)

    csv_writer = CSVWriter(dir=dir)
    if(append == False):
        open(dir, 'w').close()
        csv_writer.initCSV(strategy, indicators, type, categories, pooling)

    sid = 0
    for asset in assets:

        strategy.params.update(start=cp.deepcopy(start_params.start), td=input_td, asset=asset)
        generator = GeneratorStrategy(cp.deepcopy(generator_params), asset, indicators)

        for i in range(0, num_intervals):
            #start new generation here
            #TODO: this is a quick fix for a cummulative off by 1 error between the strategy and generator
            generator.params.update(start=cp.deepcopy(strategy.params.start), td=1)
            print("Asset: {}, Generator start: {}, Strategy start: {}".format(strategy.params.asset, generator.params.start, strategy.params.start))

            strategy.run_backtest()
            strategy.params.increment()
            strategy.params.update(td=output_td)
            #print strategy.result["date"]
            performance = trade_performance(strategy.result)

            strategy.run_backtest()
            strategy.params.update(td=input_td)
            #print strategy.result["starting_cash"]
            weights = evaluate_weights(strategy.result, type, categories)

            dates = np.empty(0)
            price_data = np.empty((0,5))
            indicator_data = np.empty((0, len(indicators)))
            for j in range(0, input_td):
                dates = np.append(dates, [str(generator.params.start)])
                generator.run_backtest()
                #print generator.result["Price"]
                price_day = np.array([generator.result['Price'][0],
                                     generator.result['Open'][0],
                                     generator.result['Close'][0],
                                     generator.result['High'][0],
                                     generator.result['Low'][0]])
                price_day = np.expand_dims(price_day, axis=0)
                price_data = np.append(price_data, price_day, axis=0)
                #print price_data
                indicator_day = np.empty((0,0))
                for indicator_str in indicators:
                    indicator_day = np.append(indicator_day, generator.indicator_values[indicator_str])
                indicator_day = np.expand_dims(indicator_day, axis=0)
                indicator_data = np.append(indicator_data, indicator_day, axis=0)

                generator.params.increment()
                #print generator.params.start

            ####NORMALIZE the data

            for tid in range(0, input_td):
                data = np.empty((0,0))
                #Sample ID
                data = np.append(data, [sid])
                #Time Step ID
                data = np.append(data, [tid])
                #Date
                data = np.append(data, [dates[tid]])
                # Asset
                data = np.append(data, [asset])
                #Price
                data = np.append(data, price_data[tid][:])
                #Indicators(Zipline Pipeline)
                data = np.append(data, indicator_data[tid][:])
                #Performance(Inputs)
                data = np.append(data, performance[tid])
                #Weights(Outputs)
                data = np.append(data, [weights])
                #note: it might make more sense to append asset to the data
                csv_writer.appendToCSV(data)
            sid += 1

    #lazy normalization
    print("Beginning normalization")
    num_lines = sum(1 for l in open(dir))
    skip_idx = [x for x in range(1, num_lines) if x > 1]
    header_df = pd.read_csv(dir, delimiter=',', delim_whitespace=False, header=0, skiprows=skip_idx)
    df = normalize_file(dir, header_df["INPUT_VECTOR_COL"][0], header_df["OUTPUT_VECTOR_COL"][0]-1)
    header_df.to_csv(dir, index=False)
    df.to_csv(dir, mode='a', index=False)
    print("Normalization complete")


###############################################
######GENERATOR/PIPELINE CODE
##############################################


class AssetMaskFactor(CustomFactor):

    inputs = [USEquityPricing.close]
    window_length = 14
    asset = None

    def compute(self, today, assets, out, close):
        out[:] = [1 if asset == self.asset else 0 for asset in assets]

class GeneratorStrategy(Strategy):

    def __init__(self, params, asset, indicators):
        super(GeneratorStrategy, self).__init__('generator', params)
        self.asset = asset
        self.indicators = indicators
        self.indicator_values = None
        self.prices = []
        self.pipeline = None

    def initialize(self):
        def initialize(context):
            self.indicator_values = {}
            context.asset = symbol(self.asset)
            context.pipeline = self.pipeline = Pipeline()
            attach_pipeline(context.pipeline, 'pipeline')
            maskFactor = AssetMaskFactor()
            maskFactor.asset = symbol(self.asset)
            mask = (maskFactor > 0)
            context.pipeline.set_screen(mask)
            for indicator_str in self.indicators:
                factor = None
                if indicator_str == 'EMA':
                    factor = ExponentialWeightedMovingAverage(inputs=[USEquityPricing.close], window_length=14, decay_rate=1.0)
                elif indicator_str == 'LongEMA':
                    factor = ExponentialWeightedMovingAverage(inputs=[USEquityPricing.close], window_length=42, decay_rate=1.0)
                elif indicator_str == 'RSI':
                    factor = RSI()
                elif indicator_str == 'PercentK':
                    factor = PercentK(14)
                elif indicator_str == 'PercentD':
                    factor = PercentD(14, 3)
                elif indicator_str == 'SlowPercentD':
                    factor = SlowPercentD(14, 3, 3)
                elif indicator_str == 'Momentum':
                    factor = Momentum(4)
                elif indicator_str == 'ROC':
                    factor = ROC(4)
                elif indicator_str == 'PercentR':
                    factor = PercentR(14)
                elif indicator_str == 'AccumulationDistribution':
                    factor = AccumulationDistribution(252) #needs around a year of recursion for accuracy
                elif indicator_str == 'ChalkinOscillator':
                    factor = ChalkinOscillator(252) #uses accumulationdistribution
                elif indicator_str == 'CommoditiesChannelIndex':
                    factor = CommoditiesChannelIndex(7)
                elif indicator_str == 'Disparity5':
                    factor = Disparity(5)
                elif indicator_str == 'Disparity10':
                    factor = Disparity(10)
                else:
                    continue
                factor.mask = mask
                context.pipeline.add(factor, indicator_str)

        return initialize

    def handle_data(self):
        def handle_data(context, data):
            record(Price=data.current(context.asset, 'price'))
            record(Open=data.current(context.asset, "open"))
            record(Close=data.current(context.asset, "close"))
            record(Low = data.current(context.asset, "low"))
            record(High = data.current(context.asset, "high"))
        return handle_data

    def analyze(self):
        def analyze(context, perf):
            for indicator_str in self.indicators:
                self.indicator_values[indicator_str] = pipeline_output('pipeline')[indicator_str].as_matrix()
            #print pipeline_output('pipeline')
            #print self.indicator_values
        return analyze

#########################################################
#####EVALUATION FUNCTIONS
#########################################################

def trade_performance(performance):

    tc = get_calendar('NYSE')
    performance = reindex(performance)
    trade_dates = [(performance['trades'][i], performance['date'][i]) \
                   for i in range(0, len(performance.index)) if check_clean(performance['trades'][i])]
    tp = np.zeros(len(performance.index))
    for trade in trade_dates:
        trade = (trade[0], tc.next_open(tc.next_open(trade[1])).replace(hour=0, minute=0))
        diff = (performance['starting_cash'][trade[1]] - performance['starting_cash'][trade[0]]) / performance['starting_cash'][trade[0]]
        index = performance.index.get_loc(trade[0])
        tp[index] = diff
    return tp


def evaluate_weights(result, type, categories):

    if type == 'regression':
        return reg_weight_eval(result)
    elif type == 'classification':
        return cat_weight_eval(result, categories)


def cat_weight_eval(result, num_cat):
    performance = np.empty((0, 0))
    bins = np.linspace(-1, 1, num_cat + 1)

    principal, final = result['starting_cash'][0], result['starting_cash'][-1]
    percent_change = (float(final - principal) / float(principal)) * 100.0
    #NOTE: CHANGE THE FOLLOWING VALUE BASED ON THE TOLERANCE YOU WANT FOR THE NETWORK
    range = 3.0
    score = round(force_within_range(percent_change / range, -0.99, 0.99),2)
    #experiment with right
    score = np.digitize(score, bins, right=True) - 1
    print("Score output: {}".format(score))
    performance = np.append(performance, [score])

    return performance


def reg_weight_eval(result):

    performance = np.empty((0, 0))
    principal, final = result['starting_cash'][0], result['starting_cash'][-1]
    percent_change = (float(final - principal) / float(principal)) * 100.0
    score = round(force_within_range(percent_change / 10.0, -1, 1),2)
    performance = np.append(performance, [score])

    return performance

###########################################################
####HELPERS
###########################################################

def pool_data(price_data):
    reduced = skimage.measure.block_reduce(price_data, (4,), np.mean)
    return reduced

def convert_to_angles(raw_data):
    diff = raw_data[1:] - raw_data[:-1]
    angles = np.arctan(diff) / (np.pi / 2)
    return np.round(angles, 3)

def check_clean(time):
    return time is not None and not pd.isnull(time)

def reindex(result):
    result.index = result.index.map(lambda t: t.replace(hour=0, minute=0))
    return result

def force_within_range(num, min, max):
    if num <= min:
        return min
    elif num >= max:
        return max
    else:
        return num