#!/bin/python3
# Zoraver Kang (zkang@wpi.edu)
from collections import defaultdict
import pandas as pd
import math
import sys
from sklearn.preprocessing import MinMaxScaler


def stats_by_stock(stocks, series):
    means = defaultdict(lambda: 0)
    stds = defaultdict(lambda: 0)
    counts = defaultdict(lambda: 0)

    for i in range(0, len(stocks)):
        means[stocks[i]] += series[i]
        counts[stocks[i]] += 1

    for key in means:
        means[key] /= counts[key]

    for i in range(0, len(stocks)):
        stds[stocks[i]] += math.pow(series[i] - means[stocks[i]], 2)

    for key in stds:
        stds[key] = math.sqrt(stds[key] / (counts[key] - 1))

    return means, stds


def normalize_series(means, stds, stocks, series):
    for i in range(0, len(stocks)):
        series[i] = (series[i] - means[stocks[i]]) / stds[stocks[i]]
    return series


def normalize_df(df, start, end, strategy=normalize_series):
    for i in range(start, end):
        column = df.iloc[:, [i]]
        scaler = MinMaxScaler(feature_range=(-1,1))
        scaler.fit(column)
        df.iloc[:, [i]] = scaler.transform(column)
            # print(column, series)
            # means, stds = stats_by_stock(stocks, series)
            # df[column] = strategy(means, stds, stocks, series)
    return df

    
def normalize_file(file_path, start, end, strategy=normalize_series):
    df = pd.read_csv(file_path, delimiter=',', delim_whitespace=False, header=2)
    return normalize_df(df, start, end, strategy)


if __name__ == '__main__':
    if len(sys.argv) >= 3:
        print(normalize_file(sys.argv[1], sys.argv[2:]))
