import zipline
import numpy as np
import pandas as pd


class Bundle(object):

    def __init__(self, name):
        self.name = name
        self.register()

    def get_ingest_func(self, start=None, end=None):
        pass

    def register(self):
        if self.name not in zipline.data.bundles.bundles:
            zipline.data.bundles.register(self.name, self.get_ingest_func())

    def unregister(self):
        zipline.data.bundles.unregister(self.name)

    def ingest(self):
        zipline.data.bundles.ingest(self.name)


# TODO: Add support for daily_bar_writer
class CSVBundle(Bundle):

    def __init__(self, name, symbols, get_csv, time_delta):
        super(Bundle, self).__init__(name)
        self.symbols = symbols
        self.get_csv = get_csv
        self.time_delta = time_delta

    def get_ingest_func(self, start=None, end=None):

        tu_symbols = tuple(self.symbols)

        def ingest(environ,
                   asset_db_writer,
                   minute_bar_writer,
                   daily_bar_writer,
                   adjustment_writer,
                   calendar,
                   start_session,
                   end_session,
                   cache,
                   show_progress,
                   output_dir):
            df_metadata = pd.DataFrame(np.empty(len(tu_symbols), dtype=[
                ('start_date', 'datetime64[ns]'),
                ('end_date', 'datetime64[ns]'),
                ('auto_close_date', 'datetime64[ns]'),
                ('symbol', 'object'),
            ]))

            csv_list = []
            num_csv = 0
            for S in tu_symbols:
                df_data = pd.read_csv(self.get_csv(S), index_col='timestamp', parse_dates=True).sort_index()

                df_data['volume'] = df_data['volume'] / 1000
                csv_list.append((num_csv, df_data))
                end_date = df_data.index[-1]

                df_metadata.iloc[num_csv] = df_data.index[0], end_date, end_date + self.time_delta, S

                num_csv += 1

            minute_bar_writer.write(csv_list, show_progress=show_progress)

            df_metadata['exchange'] = "YAHOO"

            symbol_map = pd.Series(df_metadata.symbol.index, df_metadata.symbol)

            asset_db_writer.write(equities=df_metadata)

            adjustment_writer.write()

        return ingest
