from zipline.pipeline import CustomFactor, CustomClassifier, CustomFilter
from zipline.pipeline.data import USEquityPricing

import numpy as np
from numpy import nanmax, nanmin


#TODO: relocate all code to helpers that achieve calculations, then export to the pipeline


def PercentK(window_length):
    percentk = _PercentK(window_length=window_length)
    return percentk

class _PercentK(CustomFactor):
    inputs = [USEquityPricing.high, USEquityPricing.low, USEquityPricing.close]

    def compute(self, today, assets, out, highs, lows, close):
        #print(percentk(highs, lows, close))
        out[:] = percentk(highs, lows, close)

def PercentD(bars_back, num_back):
    percentd = _PercentD(window_length = bars_back+num_back)
    percentd.num_back = num_back
    return percentd

class _PercentD(CustomFactor):
    inputs = [USEquityPricing.high, USEquityPricing.low, USEquityPricing.close]

    def compute(self, today, assets, out, highs, lows, close):
        out[:] = percentd(highs, lows, close, self.num_back)

def SlowPercentD(bars_back, reg_back, slow_back):
    slowpercentd = _SlowPercentD(window_length = bars_back+reg_back+slow_back)
    slowpercentd.reg_back = reg_back
    slowpercentd.slow_back = slow_back
    return slowpercentd

class _SlowPercentD(CustomFactor):
    inputs = [USEquityPricing.high, USEquityPricing.low, USEquityPricing.close]

    def compute(self, today, assets, out, highs, lows, close):
        total = None
        for i in range(0, self.slow_back):
            di = percentd(highs[self.slow_back-i:-(1+i)], lows[self.slow_back-i:-(1+i)], close[self.slow_back-i:-(1+i)], self.reg_back)
            if total is None:
                total = di
            else:
                total = np.add(total, di)
        out[:] = total/self.slow_back

def Momentum(window_length):
    momentum = _Momentum(window_length = window_length)
    return momentum

class _Momentum(CustomFactor):
    inputs=[USEquityPricing.close]

    def compute(self, today, assets, out, close):
        out[:] = close[-1] - close[0]

def ROC(window_length):
    roc = _ROC(window_length=window_length)
    return roc

class _ROC(CustomFactor):
    inputs=[USEquityPricing.close]

    def compute(self, today, assets, out, close):
        out[:] = (close[-1] / close[0]) * 100

def PercentR(window_length):
    percentr = _PercentR(window_length=window_length)
    return percentr

class _PercentR(CustomFactor):
    inputs=[USEquityPricing.high, USEquityPricing.low, USEquityPricing.close]

    def compute(self, today, assets, out, highs, lows, close):
        hh = nanmax(highs, axis=0)
        ll = nanmin(lows, axis=0)
        c=close[-1]
        out[:] = ((hh-c)/(hh-ll)) * 100

def AccumulationDistribution(window_length):
    ad = _AccumulationDistribution(window_length=window_length)
    return ad

class _AccumulationDistribution(CustomFactor):
    inputs = [USEquityPricing.high, USEquityPricing.low, USEquityPricing.close, USEquityPricing.volume]

    def compute(self, today, assets, out, highs, lows, close, volume):
        #print(accumulation_distribution(highs, lows, close, volume)[-1])
        out[:] = accumulation_distribution(highs, lows, close, volume)[-1]



def ChalkinOscillator(window_length):
    co = _ChalkinOscillator(window_length=window_length)
    return co

class _ChalkinOscillator(CustomFactor):
    inputs=[USEquityPricing.high, USEquityPricing.low, USEquityPricing.close, USEquityPricing.volume]

    def compute(self, today, assets, out, highs, lows, close, volume):
        ad = accumulation_distribution(highs, lows, close, volume)
        ema3 = exponential_moving_average(ad, 3)
        ema10 = exponential_moving_average(ad, 10)
        out[:] = ema3 - ema10

def CommoditiesChannelIndex(period):
    cci = _CommoditiesChannelIndex(window_length=period+1)
    cci.period = period
    return cci

#thanks to kylejusticemagnuson
class _CommoditiesChannelIndex(CustomFactor):
    inputs=[USEquityPricing.high, USEquityPricing.low, USEquityPricing.close]

    def compute(self, today, assets, out, highs, lows, close):
        out[:] = commodities_channel_index(highs, lows, close, self. period)[-1]

def Disparity(period):
    disparity = _Disparity(window_length=period+1)
    disparity.period = period
    return disparity

class _Disparity(CustomFactor):
    inputs=[USEquityPricing.close]

    def compute(self, today, assets, out, close):
        out[:] = disparity(close, self.period)

def disparity(close, period):
    return (close[-1] / simple_moving_average(close, period)[-1]) * 100

def Votality(period):
    votality = _Votality(window_length=period)
    return votality

class _Votality(CustomFactor):
    inputs=[USEquityPricing.close]

    def compute(self, today, assets, out, close):
        out[:] = close.std() / close.var()

def commodities_channel_index(highs, lows, closes, period):
    tp = typical_price(highs, lows, closes)
    sma = simple_moving_average(tp, period)
    cci = ((tp - sma) /
           (0.015 * np.mean(np.absolute(tp - np.mean(tp)))))
    return cci

# thanks to kylejusticemagnuson
def typical_price(highs, lows, closes):
    tp = [(highs[idx] + lows[idx] + closes[idx])/3 for idx in range(0, len(closes))]
    return (np.array(tp).T)[0]

def simple_moving_average(data, period):
    sma = [np.mean(data[idx - period:idx]) for idx in range(period, len(data))]
    return fill_for_noncomputable_vals(data, sma)

#todo: add extra function to analyze entire array and generate emas
#credits to kylejusticemagnuson on github
def exponential_moving_average(data, period):
    w = 2/float(period+1)
    ema_top = 0
    ema_bottom = 0
    for idx in range(0,period):
        ema_top += ((1-w)**idx) * data[-(idx+1)]
        ema_bottom += (1 - w)**idx
    return ema_top/ema_bottom

def accumulation_distribution(highs, lows, closes, volumes):
    ad = np.zeros(len(closes))
    for idx in range(1, len(closes)):
        ad[idx] = \
            money_flow_volume(highs[idx], lows[idx], closes[idx], volumes[idx]) + ad[idx-1]
    return ad

def money_flow_volume(h, l, c, v):
    return money_flow_multiplier(h, l, c) * v

def money_flow_multiplier(h, l, c):
    return ((c-l)-(h-c))/(h-l)

def percentd(highs, lows, close, num_back):
    #print "close ", close #for debug purposes only
    total = None
    for i in range(0, num_back):
        ki = percentk(highs[num_back-i:-(1+i)], lows[num_back-i:-(1+i)], close[num_back-i:-(1+i)])
        if total is None:
            total = ki
        else:
            total = np.add(total, ki)
    return total / num_back

def percentk(highs, lows, close):
    hh = nanmax(highs, axis=0)
    ll = nanmin(lows, axis=0)
    c=close[-1]
    return ((c-ll)/(hh-ll)) * 100

#credits to kylejusticemagnuson on github
def fill_for_noncomputable_vals(input_data, result_data):
    non_computable_values = np.repeat(
        np.nan, len(input_data) - len(result_data)
        )
    filled_result_data = np.append(non_computable_values, result_data)
    return filled_result_data