#import datetime
from zipline.utils.calendars import get_calendar
from datetime import datetime, timedelta
import pandas as pd
import pytz
import copy as cp


class StrategyParameters(object):

    def __init__(self, start, td, principal, frequency, bundle, asset):
        self.start = start
        self.td = td
        self.end = self.set_end()
        self.principal = principal
        self.frequency = frequency
        self.bundle = bundle
        self.asset = asset

    def update(self, start=None, td=None, principal=None, frequency=None, bundle=None, asset=None):
        if start:
            self.start = start
            self.end = self.set_end()
        if td:
            self.td = td
            self.end = self.set_end()
        if principal != None:
            self.principal = principal
        if frequency:
            self.frequency = frequency
        if bundle:
            self.bundle = bundle
        if asset:
            self.asset = asset
        return self

    def copy(self):
        pass

    def increment(self, principal=None):
        pass

    def decrement(self, principal=None):
        pass

    def set_end(self, principal=None):
        pass

    @staticmethod
    def parse_time(year, month, day):
        return datetime.datetime(year, month, day, 0, 0, 0, 0, pytz.timezone('US/Eastern'))


class DailyStrategyParameters(StrategyParameters):

    def __init__(self, start, td, principal, frequency, bundle, asset):
        super(DailyStrategyParameters, self).__init__(start, td, principal, frequency, bundle, asset)

    def increment(self, principal=None):
        tc = get_calendar('NYSE')
        if principal:
            self.principal = principal
        self.start = self.end
        ts_end = pd.Timestamp(self.end)
        ts_end = tc.next_open(ts_end)
        for i in range(0, self.td):
            ts_end = tc.next_open(ts_end)
        ts_end = ts_end.replace(hour=0, minute=0)
        self.end = ts_end.to_datetime()
        #print self.start, self.end
        return self

    def decrement(self, principal=None):
        tc = get_calendar('NYSE')
        if principal:
            self.principal = principal
        self.end = self.start
        ts_start = pd.Timestamp(self.start)
        ts_start = tc.previous_open(ts_start)
        for i in range(0, self.td):
            ts_start = tc.previous_open(ts_start)
        ts_start = ts_start.replace(hour=0, minute=0)
        self.start = ts_start.to_datetime()
        return self

    #TODO: SHIFT LEFT AND SHIFT RIGHT MIGHT NEED REPAIRS
    def shift_left(self, td):
        tc = get_calendar('NYSE')
        ts_start, ts_end = pd.Timestamp(self.start), pd.Timestamp(self.end)
        vd = 0
        while vd < td:
            ts_start = tc.previous_open(ts_start)
            ts_end = tc.previous_open(ts_end)
            vd += 1
        ts_start, ts_end = ts_start.replace(hour=0, minute=0), ts_end.replace(hour=0, minute=0)
        self.start, self.end = ts_start.to_datetime(), ts_end.to_datetime()
        return self

    def shift_right(self, td):
        tc = get_calendar('NYSE')
        ts_start, ts_end = pd.Timestamp(self.start), pd.Timestamp(self.end)
        vd = 0
        while vd < td:
            ts_start = tc.next_open(ts_start)
            ts_end = tc.next_open(ts_end)
            vd += 1
        ts_start, ts_end = ts_start.replace(hour=0, minute=0), ts_end.replace(hour=0, minute=0)
        self.start, self.end = ts_start.to_datetime(), ts_end.to_datetime()
        return self

    def set_end_old(self):
        tc = get_calendar('NYSE')
        end = self.start + timedelta(days=self.td)
        ts_end = pd.Timestamp(end)
        vd = sum(tc.is_session(pd.Timestamp(self.start + timedelta(days = i))) for i in range(0, self.td))
        while vd < self.td:
            ts_end = tc.next_open(ts_end)
            vd += 1
        ts_end = ts_end.replace(hour=0, minute=0)
        return ts_end.to_datetime()

    def set_end(self):
        tc = get_calendar('NYSE')
        ts_end = pd.Timestamp(self.start)
        ts_end = tc.next_open(ts_end)
        for i in range(0, self.td):
            ts_end = tc.next_open(ts_end)
        ts_end = ts_end.replace(hour=0, minute=0)
        self.end = ts_end.to_datetime()
        return self.end


class MinuteStrategyParameters(StrategyParameters):

    def __init__(self, start, td, principal, frequency, bundle, asset):
        super(MinuteStrategyParameters, self).__init__(start, td, principal, frequency, bundle, asset)

    def increment(self, principal=None):
        tc = get_calendar('NYSE')
        if principal:
            self.principal = principal
        self.start = self.end
        self.end = self.end + timedelta(minutes=self.td)
        ts_end = pd.Timestamp(self.end)
        vd = sum(tc.is_session(pd.Timestamp(self.start + timedelta(minutes = i))) for i in range(0, self.td))
        while vd < self.td:
            ts_end = tc.next_minute(ts_end)
            vd += 1
        #NOTE: THIS LINE MAY NEED REPLACEMENT
        #ts_end = ts_end.replace(hour=0, minute=0)
        self.end = ts_end.to_datetime()
        return self

    def decrement(self, principal=None):
        tc = get_calendar('NYSE')
        if principal:
            self.principal = principal
        self.end = self.start
        self.start = self.start - timedelta(minutes=self.td)
        ts_start = pd.Timestamp(self.start)
        vd = sum(tc.is_session(pd.Timestamp(self.start + timedelta(minutes = i))) for i in range(0, self.td))
        while vd < self.td:
            ts_start = tc.previous_minute(ts_start)
            vd += 1
        #NOTE: THIS LINE MIGHT NEED REPLACEMENT
        #ts_start = ts_start.replace(hour=0, minute=0)
        self.start = ts_start.to_datetime()
        return self

    def shift_left(self, td):
        tc = get_calendar('NYSE')
        ts_start, ts_end = pd.Timestamp(self.start), pd.Timestamp(self.end)
        vd = 0
        while vd < td:
            ts_start = tc.previous_minute(ts_start)
            ts_end = tc.previous_minute(ts_end)
            vd += 1
        ts_start, ts_end = ts_start.replace(hour=0, minute=0), ts_end.replace(hour=0, minute=0)
        self.start, self.end = ts_start.to_datetime(), ts_end.to_datetime()
        return self

    def shift_right(self, td):
        tc = get_calendar('NYSE')
        ts_start, ts_end = pd.Timestamp(self.start), pd.Timestamp(self.end)
        vd = 0
        while vd < td:
            ts_start = tc.next_minute(ts_start)
            ts_end = tc.next_minute(ts_end)
            vd += 1
        ts_start, ts_end = ts_start.replace(hour=0, minute=0), ts_end.replace(hour=0, minute=0)
        self.start, self.end = ts_start.to_datetime(), ts_end.to_datetime()
        return self

    def set_end(self):
        tc = get_calendar('NYSE')
        end = self.start + timedelta(minutes=self.td)
        ts_end = pd.Timestamp(end)
        vd = sum(tc.is_session(pd.Timestamp(self.start + timedelta(minutes = i))) for i in range(0, self.td))
        while vd < self.td:
            ts_end = tc.next_minute(ts_end)
            vd += 1
        #NOTE: THIS LINE MIGHT NEED REPLACEMENT
        #ts_end = ts_end.replace(hour=0, minute=0)
        return ts_end.to_datetime()