from keras.models import Sequential
from keras.layers import Dense, LeakyReLU
from keras.regularizers import l2
from keras import optimizers
from nn.cnn.csv_reader.data_retrieve import *
from nn.cnn.neural_networks.abs_network import *


# Hack to mitigate "AttributeError: 'LeakyReLU' object has no attribute '__name__'"
# Inspired by snippet from decaelus on GitHub
class FixedLeakyReLU(LeakyReLU):
    def __init__(self, alpha=0.3, **kwargs):
        self.__name__ = "FixedLeakyReLU"
        super(FixedLeakyReLU, self).__init__(alpha, **kwargs)


class TestNeuralNetwork(RegressionNeuralNetwork):

    def __init__(self):
        super(TestNeuralNetwork, self).__init__()

    def build_network(self, input_shape, output_shape):
        model = Sequential()

        model.add(Dense(input_shape, input_shape=(input_shape,), kernel_initializer='glorot_uniform', activation='tanh',
                        use_bias=True, bias_regularizer=l2(0.1)))
        model.add(Dense(input_shape, kernel_initializer='glorot_uniform', activation='tanh',
                        use_bias=True, bias_regularizer=l2(0.1)))
        model.add(Dense(input_shape, kernel_initializer='glorot_uniform', activation='sigmoid',
                        use_bias=True, bias_regularizer=l2(0.1)))
        model.add(Dense(input_shape, kernel_initializer='glorot_uniform', activation='tanh',
                        use_bias=True, bias_regularizer=l2(0.1)))

        model.add(Dense(1, kernel_initializer='glorot_uniform', activation='linear', use_bias=True))
        # model.add(Dense(1,input_shape=(input_shape, ), kernel_initializer='glorot_uniform', activation='tanh'))

        sgd = optimizers.SGD(lr=0.01)
        model.compile(optimizer=sgd,
                      loss='mse',
                      metrics=['accuracy'])

        self.model = model


class CategoricalNeuralNetwork(ClassificationNeuralNetwork):
    def __init__(self):
        super(CategoricalNeuralNetwork, self).__init__()

    def build_network(self, input_shape, output_shape):
        model = Sequential()

        model.add(Dense(input_shape, input_shape=(input_shape,), kernel_initializer='glorot_uniform', activation='tanh',
                        use_bias=True))
        model.add(Dense(2 * input_shape, kernel_initializer='glorot_uniform', activation=FixedLeakyReLU(),
                        use_bias=True, kernel_regularizer=l2(0.001)))
        model.add(Dense(2 * input_shape, kernel_initializer='glorot_uniform', activation='tanh',
                        use_bias=True, kernel_regularizer=l2(0.001)))
        model.add(Dense(2 * input_shape, kernel_initializer='glorot_uniform', activation=FixedLeakyReLU(),
                        use_bias=True, kernel_regularizer=l2(0.001)))
        model.add(Dense(2 * input_shape, kernel_initializer='glorot_uniform', activation='tanh',
                        use_bias=True, kernel_regularizer=l2(0.001)))
        model.add(Dense(2 * input_shape, kernel_initializer='glorot_uniform', activation=FixedLeakyReLU(),
                        use_bias=True, kernel_regularizer=l2(0.001)))
        # #not sure whether to keep these two lines or not
        # model.add(Dense(2*input_shape, kernel_initializer='glorot_uniform', activation='tanh',
        #                 use_bias=True, bias_regularizer=l2(0.01), kernel_regularizer=l2(0.01)))
        # model.add(Dense(2*input_shape, kernel_initializer='glorot_uniform', activation='relu',
        #                 use_bias=True, bias_regularizer=l2(0.01), kernel_regularizer=l2(0.01)))
        # model.add(Dense(2 * input_shape, kernel_initializer='glorot_uniform', activation='tanh',
        #                 use_bias=True, bias_regularizer=l2(0.01), kernel_regularizer=l2(0.01)))
        # model.add(Dense(2 * input_shape, kernel_initializer='glorot_uniform', activation='relu',
        #                 use_bias=True, bias_regularizer=l2(0.01), kernel_regularizer=l2(0.01)))
        # model.add(Dense(2 * input_shape, kernel_initializer='glorot_uniform', activation='tanh',
        #                 use_bias=True, bias_regularizer=l2(0.01), kernel_regularizer=l2(0.01)))
        # model.add(Dense(2 * input_shape, kernel_initializer='glorot_uniform', activation='relu',
        #                 use_bias=True, bias_regularizer=l2(0.01), kernel_regularizer=l2(0.01)))
        # model.add(Dense(2 * input_shape, kernel_initializer='glorot_uniform', activation='tanh',
        #                 use_bias=True, bias_regularizer=l2(0.01), kernel_regularizer=l2(0.01)))
        # model.add(Dense(2 * input_shape, kernel_initializer='glorot_uniform', activation='relu',
        #                 use_bias=True, bias_regularizer=l2(0.01), kernel_regularizer=l2(0.01)))
        model.add(Dense(2 * input_shape, kernel_initializer='glorot_uniform', activation='tanh',
                        use_bias=True, kernel_regularizer=l2(0.001)))
        model.add(Dense(2 * input_shape, kernel_initializer='glorot_uniform', activation='relu',
                        use_bias=True, kernel_regularizer=l2(0.001)))

        model.add(Dense(output_shape, kernel_initializer='glorot_uniform', activation='softmax', use_bias=True))

        sgd = optimizers.SGD(lr=0.01)
        model.compile(optimizer=sgd,
                      loss='categorical_crossentropy',
                      metrics=['accuracy'])

        self.model = model


class EMA2CNN(ClassificationNeuralNetwork):
    def __init__(self):
        super(EMA2CNN, self).__init__()

    def build_network(self, input_shape, output_shape):
        model = Sequential()

        model.add(Dense(input_shape, input_shape=(input_shape,), kernel_initializer='glorot_uniform', activation='tanh',
                        use_bias=True))
        model.add(Dense(2 * input_shape, kernel_initializer='glorot_uniform', activation=FixedLeakyReLU(),
                        use_bias=True, kernel_regularizer=l2(0.001)))
        model.add(Dense(2 * input_shape, kernel_initializer='glorot_uniform', activation='tanh',
                        use_bias=True, kernel_regularizer=l2(0.001)))
        model.add(Dense(2 * input_shape, kernel_initializer='glorot_uniform', activation=FixedLeakyReLU(),
                        use_bias=True, kernel_regularizer=l2(0.001)))
        model.add(Dense(2 * input_shape, kernel_initializer='glorot_uniform', activation='tanh',
                        use_bias=True, kernel_regularizer=l2(0.001)))
        model.add(Dense(2 * input_shape, kernel_initializer='glorot_uniform', activation=FixedLeakyReLU(),
                        use_bias=True, kernel_regularizer=l2(0.001)))
        model.add(Dense(2 * input_shape, kernel_initializer='glorot_uniform', activation='tanh',
                        use_bias=True, kernel_regularizer=l2(0.001)))
        model.add(Dense(2 * input_shape, kernel_initializer='glorot_uniform', activation='relu',
                        use_bias=True, kernel_regularizer=l2(0.001)))

        model.add(Dense(output_shape, kernel_initializer='glorot_uniform', activation='softmax', use_bias=True))

        sgd = optimizers.SGD(lr=0.01)
        model.compile(optimizer=sgd,
                      loss='categorical_crossentropy',
                      metrics=['accuracy'])

        self.model = model


def main():
    ts = TrainingSet('data/ema_full_2.csv')
    tnn = TestNeuralNetwork()
    cnn = CategoricalNeuralNetwork()

    # tnn.train_model(ts)
    cnn.train_model(ts)
    # cnn.model.save('../../data/models/test.h5')
    # cnn.model.save_weights('../../data/models/test_weights.h5')
    cnn.save('data/models/ema_full_2_weights.h5', 'data/models/ema_full_2_manifest.pickle')


if __name__ == '__main__':
    main()
