# Copyright [2018, +inf), Zoraver Kang (zkang@wpi.edu) and Remy Kaldawy

import pickle
import unittest
import os

from nn.cnn.csv_reader.data_retrieve import TrainingSet


class Manifest(object):

    def __init__(self, traning_path: str, weights_path: str, network_type: type):
        self.training_path = traning_path
        self.weights_path = weights_path
        self.network_type = network_type
        with open(self.training_path, 'r') as file:
            file.readline()  # we only care about the third line
            file.readline()
            header = file.readline().rpartition("Price")[2].split(',')[1:]
            self.indicators = []
            self.strategies = []
            for s in header:
                if not s.endswith("Performance"):
                    self.indicators.append(s)
                else:
                    break
            for s in header[len(self.indicators):]:
                if s.endswith("Performance"):
                    self.strategies.append(s.replace("Performance", ""))

    def __eq__(self, other):
        return self.training_path == other.training_path and \
               self.weights_path == other.weights_path and \
               self.network_type == other.network_type and \
               self.indicators == other.indicators and \
               self.strategies == other.strategies

    def save(self, path):
        with open(path, 'wb') as file:
            pickle.dump(self, file)

    def get_network(self):
        ts = TrainingSet(self.training_path)
        network = self.network_type()
        network.build_network(ts.getInputSize(), ts.getOutputSize())
        network.model.load_weights(self.weights_path)
        return network

    @staticmethod
    def load(path):
        with open(path, 'rb') as file:
            return pickle.load(file)


class Test(unittest.TestCase):

    def test_load_save_cycle(self):
        from nn.cnn.neural_networks.networks import CategoricalNeuralNetwork
        a = Manifest('../../data/doodeedoodee.csv', '../../data/models/test_weights.h5', CategoricalNeuralNetwork)
        a.save("test.pickle")
        b = Manifest.load("test.pickle")
        os.remove("test.pickle")
        self.assertEqual(a, b)

    def test_indicators(self):
        from nn.cnn.neural_networks.networks import CategoricalNeuralNetwork
        a = Manifest('../../data/doodeedoodee.csv', '../../data/models/test_weights.h5', CategoricalNeuralNetwork)
        self.assertEqual(a.indicators,
                         ['EMA', 'LongEMA', 'RSI', 'PercentK', 'PercentD', 'SlowPercentD', 'Momentum', 'ROC',
                          'PercentR',
                          'AccumulationDistribution', 'ChalkinOscillator', 'CommoditiesChannelIndex', 'Disparity5',
                          'Disparity10'])

    def test_strategies(self):
        from nn.cnn.neural_networks.networks import CategoricalNeuralNetwork
        a = Manifest('../../data/doodeedoodee.csv', '../../data/models/test_weights.h5', CategoricalNeuralNetwork)
        self.assertEqual(a.strategies, ["SimpleEMAStrategy"])

    def test_get_network(self):
        from nn.cnn.neural_networks.networks import CategoricalNeuralNetwork
        a = Manifest('../../data/doodeedoodee.csv', '../../data/models/test_weights.h5', CategoricalNeuralNetwork)
        self.assertIsInstance(a.get_network(), CategoricalNeuralNetwork)
